@@ -1,12 +1,13 @@
 /*
- * tools/lib/liblvm.h
+ * Tools/lib/liblvm.h
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March-June 1997
  * January 1998
  * January,February,July 1999
  * February,March 2000
+ * March,June 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -22,7 +23,7 @@
  * You should have received a copy of the GNU Library General Public
  * License along with this LVM library; if not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA 
+ * MA 02111-1307, USA
  *
  */
 
@@ -32,6 +33,9 @@
  *   03/01/1999 - port to libc6
  *   07/02/1999 - conditional data conversion macros
  *   05/07/1999 - rearanged includes due to 2.3.x changes
+ *   05/04/2001 - changed pv_move_pes() and pv_move_pe() prototypes
+ *   09/04/2001 - added prototype for pv_get_partitioned_whole
+ *   26/06/2001 - fixed pv_get_size() typecast on Alpha
  *
  */
 
@@ -75,9 +79,7 @@
 #include <dirent.h>
 #include <stdarg.h>
 #include <malloc.h>
-#ifndef __sparc__
-#  include <string.h>
-#endif
+#include <string.h>
 #include <ctype.h>
 #include <time.h>
 #ifdef _G_LSEEK64
@@ -93,13 +95,18 @@ int llseek ( unsigned int, unsigned long long, unsigned int);
 
 #include <lvm.h>
 
-#define	SECTOR_SIZE	512
-#define	BLKGETSIZE	_IO(0x12,96)	/* return device size */ 
+#include "lvm_log.h"
+#include "lvm_config.h"
+
+/* FIXME: get rid of these from lib code */
+#define FALSE	0
+#define TRUE	1
+
+extern struct config_file *config_file;
+
+#define	BLKGETSIZE	_IO(0x12,96)	/* return device size */
 #define	BLKRASET	_IO(0x12,98)	/* Set read ahead */
 #define	EXT2_SUPER_MAGIC	0xEF53
-#ifndef BLOCK_SIZE
-#define	BLOCK_SIZE	1024
-#endif
 
 #define	LVM_ID		"HM"      /* Identifier PV (id in pv_t) */
 #define	EXPORTED	"PV_EXP"  /* Identifier exported PV (system_id in pv_t) */
@@ -115,6 +122,36 @@ int llseek ( unsigned int, unsigned long long, unsigned int);
 #define	MIN_PART		1
 #define	MAX_PART	 	15
 
+/* some metadata on the disk need to be aligned */
+#define LVM_VGDA_ALIGN 4096UL
+
+/* base of PV structure in disk partition */
+#define	LVM_PV_DISK_BASE	0L
+
+/* size reserved for PV structure on disk */
+#define	LVM_PV_DISK_SIZE	1024L
+
+/* base of VG structure in disk partition */
+#define	LVM_VG_DISK_BASE round_up(LVM_PV_DISK_BASE + LVM_PV_DISK_SIZE, \
+                                  LVM_VGDA_ALIGN)
+
+/* size reserved for VG structure */
+#define	LVM_VG_DISK_SIZE  	(8 * 512L)
+
+/* name list of physical volumes on disk */
+#define	LVM_PV_UUIDLIST_DISK_BASE round_up(LVM_VG_DISK_BASE + \
+                                           LVM_VG_DISK_SIZE, LVM_VGDA_ALIGN)
+
+/* now for the dynamically calculated parts of the VGDA */
+#define	LVM_LV_DISK_OFFSET(a, b) ((a)->lv_on_disk.base + sizeof(lv_disk_t) * b)
+
+#define	LVM_VGDA_SIZE(pv) ((pv)->pe_on_disk.base + (pv)->pe_on_disk.size)
+
+#define	LVM_PE_DISK_OFFSET(pe, pv) \
+	((pe * pv->pe_size) + pv->data_location.base)
+
+#define	LVM_PE_ALIGN		 (65536UL / SECTOR_SIZE)
+
 
 extern int errno;
 extern char *cmd;
@@ -139,7 +176,7 @@ alloc_t;
 typedef struct {
    char *dev_name;
    dev_t st_rdev;
-   short st_mode;
+   mode_t st_mode;
 } dir_cache_t;
 
 /* PE type data layer functions */
@@ -176,7 +213,9 @@ int  vg_read_with_pv_and_lv ( char *, vg_t **);
 inline int vg_reduce ( char *, pv_t *, vg_t *);
 int  vg_remove_dir_and_group_and_nodes ( char *);
 int  vg_set_extendable ( char*);
+void vg_setup_pointers_for_snapshots ( vg_t*);
 int  vg_clear_extendable ( char*);
+int setup_pe_table(pv_t *p); /* used by the vg_setup_* functions */
 int  vg_setup_for_create ( char *, vg_t *, pv_t **, int, ulong, ulong);
 int  vg_setup_for_extend ( char **, int, pv_t **, vg_t *, char **);
 int  vg_setup_for_merge ( vg_t *, vg_t *);
@@ -222,9 +261,9 @@ int    pv_get_index_by_kdev_t ( vg_t *, kdev_t);
 int    pv_get_index_by_name ( vg_t *, char *);
 kdev_t pv_get_kdev_t_by_number ( vg_t *, int);
 int    pv_get_size ( char *, struct partition *);
-int    pv_move_pes ( vg_t*, char*, char**, int, int,
+int    pv_move_pes ( vg_t*, char*, size_t, char**, int, int,
                      long*, long*, long*, int, int);
-int    pv_move_pe ( vg_t*, char*, long, long, long, long,
+int    pv_move_pe ( vg_t*, char*, size_t, long, long, long, long,
                     int, int, int, int);
 int    pv_read ( char *, pv_t **, int *);
 int    pv_read_already_red ( char *);
@@ -232,7 +271,7 @@ int    pv_read_pe ( pv_t *, pe_disk_t **);
 int    pv_read_all_pv ( pv_t ***, int);
 int    pv_read_all_pv_of_vg ( char *, pv_t ***, int);
 int    pv_read_all_pe_of_vg ( char *, pe_disk_t ***, int);
-int    pv_read_uuidlist ( char *, char **);
+int    pv_read_uuidlist ( pv_t *, char **);
 int    pv_release_pe ( vg_t *, pe_disk_t *, uint *, uint);
 int    pv_reserve_pe ( pv_t *, pe_disk_t *, uint *, pe_t *, uint, int);
 int    pv_setup_for_create ( char *, pv_t *, uint);
@@ -250,7 +289,7 @@ int    pv_write_all_pv_of_vg ( vg_t *);
 int    pv_write_uuidlist ( char *, vg_t *);
 int    pv_write_pe ( char*, pv_t *);
 int    pv_write_with_pe ( char*, pv_t *);
-
+int    pv_check_partitioned_whole(char *pv_name);
 
 /* LV functions */
 char   *lv_change_vgname ( char *, char *);
@@ -308,9 +347,19 @@ int    lv_write_all_lv ( char*, vg_t *);
 
 
 /* print debug info on stdout */
-void debug ( const char *, ...);
-void debug_enter ( const char *, ...);
-void debug_leave ( const char *, ...);
+#ifdef DEBUG
+void lvm_debug(const char *fmt, ...);
+void lvm_debug_enter(const char *fmt, ...);
+void lvm_debug_leave(const char *fmt, ...);
+
+#define debug(fmt, args...) lvm_debug(fmt, ## args)
+#define debug_enter(fmt, args...) lvm_debug_enter(fmt, ## args)
+#define debug_leave(fmt, args...) lvm_debug_leave(fmt, ## args)
+#else
+#define debug(fmt, args...)
+#define debug_enter(fmt, args...)
+#define debug_leave(fmt, args...)
+#endif
 
 /* generate nice KB/MB/... strings */
 char *lvm_show_size ( unsigned long long, size_len_t);
@@ -330,6 +379,7 @@ int system_id_check_imported ( char *);
 /* LVM locking / interrupt masking etc. */
 int    lvm_check_chars ( char*);
 int    lvm_check_dev ( struct stat*, int);
+int    lvm_check_devfs ();
 int    lvm_check_extended_partition ( dev_t);
 int    lvm_check_kernel_lvmtab_consistency ( void);
 int    lvm_check_partitioned_dev ( dev_t);
@@ -344,6 +394,7 @@ dir_cache_t *lvm_dir_cache_find ( char *);
 void   lvm_dont_interrupt ( int);
 int    lvm_get_col_numbers ( char *, long **);
 int    lvm_get_iop_version ( void);
+void   lvm_init(int argc, char **argv);
 void   lvm_interrupt ( void);
 int    lvm_lock ( void);
 int    lvm_partition_count ( dev_t);
@@ -367,6 +418,44 @@ int  lvm_tab_vg_check_exist ( char *, vg_t **);
 int  lvm_tab_get_free_blk_dev ( kdev_t **);
 char **lvm_tab_vg_check_exist_all_vg ( void);
 
+/* core <-> disk conversion macros */
+#if __BYTE_ORDER == __BIG_ENDIAN
+#define LVM_TO_CORE16(x) ( \
+        ((uint16_t)((((uint16_t)(x) & 0x00FFU) << 8) | \
+                    (((uint16_t)(x) & 0xFF00U) >> 8))))
+
+#define LVM_TO_DISK16(x) LVM_TO_CORE16(x)
+
+#define LVM_TO_CORE32(x) ( \
+        ((uint32_t)((((uint32_t)(x) & 0x000000FFU) << 24) | \
+                    (((uint32_t)(x) & 0x0000FF00U) << 8)  | \
+                    (((uint32_t)(x) & 0x00FF0000U) >> 8)  | \
+                    (((uint32_t)(x) & 0xFF000000U) >> 24))))
+
+#define LVM_TO_DISK32(x) LVM_TO_CORE32(x)
+
+#define LVM_TO_CORE64(x) \
+        ((uint64_t)((((uint64_t)(x) & 0x00000000000000FFULL) << 56) | \
+                    (((uint64_t)(x) & 0x000000000000FF00ULL) << 40) | \
+                    (((uint64_t)(x) & 0x0000000000FF0000ULL) << 24) | \
+                    (((uint64_t)(x) & 0x00000000FF000000ULL) <<  8) | \
+                    (((uint64_t)(x) & 0x000000FF00000000ULL) >>  8) | \
+                    (((uint64_t)(x) & 0x0000FF0000000000ULL) >> 24) | \
+                    (((uint64_t)(x) & 0x00FF000000000000ULL) >> 40) | \
+                    (((uint64_t)(x) & 0xFF00000000000000ULL) >> 56)))
+
+#define LVM_TO_DISK64(x) LVM_TO_CORE64(x)
+#elif __BYTE_ORDER == __LITTLE_ENDIAN
+#define LVM_TO_CORE16(x) x
+#define LVM_TO_DISK16(x) x
+#define LVM_TO_CORE32(x) x
+#define LVM_TO_DISK32(x) x
+#define LVM_TO_CORE64(x) x
+#define LVM_TO_DISK64(x) x
+#else
+#error "__BYTE_ORDER must be defined as __LITTLE_ENDIAN or __BIG_ENDIAN"
+#endif /* #if __BYTE_ORDER == __BIG_ENDIAN */
+
 
 /* return codes */
 #define LVM_VG_CFGBACKUP_NO_DIFF                                          100
@@ -646,7 +735,7 @@ char **lvm_tab_vg_check_exist_all_vg ( void);
 #define LVM_EVG_READ_WITH_PV_AND_LV_PV_CUR                                  367
 #define LVM_EVG_RENAME_OPEN                                                 368
 #define LVM_EVG_SETUP_FOR_CREATE_MALLOC                                     369
-#define LVM_EVG_SETUP_FOR_CREATE_PV_SIZE                                    370
+#define LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MIN                                370
 #define LVM_EVG_SETUP_FOR_CREATE_VG_NUMBER                                  371
 #define LVM_EVG_SETUP_FOR_EXTEND_MALLOC                                     372
 #define LVM_EVG_SETUP_FOR_EXTEND_MAX_PV                                     373
@@ -681,5 +770,26 @@ char **lvm_tab_vg_check_exist_all_vg ( void);
 #define LVM_EVG_WRITE_LSEEK                                                 402
 #define LVM_EVG_WRITE_OPEN                                                  403
 #define LVM_EVG_WRITE_WRITE                                                 404
+#define LVM_ELV_PV_CREATE_NAME_FROM_KDEV_T                                  405
+#define LVM_EPV_FLUSH_STAT                                                  406
+#define LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MAX				    407
+
+
+/*
+ * useful inlines
+ */
+static inline unsigned long sectors_to_k(unsigned long sect)
+{
+	if(SECTOR_SIZE > 1024) {
+		return sect * (SECTOR_SIZE / 1024);
+	}
+
+	return sect / (1024 / SECTOR_SIZE);
+}
+
+static inline unsigned long get_pe_offset(ulong p, pv_t *pv)
+{
+	return pv->pe_start + (p * pv->pe_size);
+}
 
 #endif /* #ifndef _LIBLVM_H_INCLUDE */
