@@ -1,11 +1,12 @@
 /*
  * tools/lib/lvm_check_dev.c
  *
- * Copyright (C) 1998 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1998 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * May,August,November 1998
  * February,April 1999
- * January 2000
+ * January,April 2000
+ * February 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -35,6 +36,8 @@
  *    17/03/1999 - added loop device support
  *    22/04/1999 - DAC960 support by Matthew Taylor <M.Taylor@rbgkew.org.uk>
  *    31/01/2000 - use debug_enter()/debug_leave()
+ *    26/04/2000 - added DASD major support
+ *    07/02/2001 - added COMPAQ_CISS major support
  *
  */
 
@@ -43,39 +46,7 @@
 
 int lvm_check_dev ( struct stat *stat_b, int check_mode) {
    int ret = FALSE;
-   int i = 0;
-   short major = 0;
-   dev_t valid_majors[] = {
-#ifndef SCSI_DISK0_MAJOR
-      SCSI_DISK_MAJOR,
-#else
-      SCSI_DISK0_MAJOR,
-      SCSI_DISK1_MAJOR,
-      SCSI_DISK2_MAJOR,
-      SCSI_DISK3_MAJOR,
-      SCSI_DISK4_MAJOR,
-      SCSI_DISK5_MAJOR,
-      SCSI_DISK6_MAJOR,
-      SCSI_DISK7_MAJOR,
-#endif
-      IDE0_MAJOR,
-      IDE1_MAJOR,
-      IDE2_MAJOR,
-      IDE3_MAJOR,
-#ifdef IDE4_MAJOR
-      IDE4_MAJOR,
-      IDE5_MAJOR,
-#endif
-#ifdef	DAC960_MAJOR
-      DAC960_MAJOR,
-      DAC960_MAJOR+1,
-      DAC960_MAJOR+2,
-      DAC960_MAJOR+3,
-      DAC960_MAJOR+4,
-      DAC960_MAJOR+5,
-      DAC960_MAJOR+6,
-      DAC960_MAJOR+7,
-#endif
+   dev_t unpartitioned_majors[] = {
 #ifdef LOOP_MAJOR
       LOOP_MAJOR,
 #endif
@@ -84,47 +55,19 @@ int lvm_check_dev ( struct stat *stat_b, int check_mode) {
 #endif
 #ifdef NBD_MAJOR
       NBD_MAJOR,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR
-      COMPAQ_SMART2_MAJOR,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR1
-      COMPAQ_SMART2_MAJOR1,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR2
-      COMPAQ_SMART2_MAJOR2,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR3
-      COMPAQ_SMART2_MAJOR3,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR4
-      COMPAQ_SMART2_MAJOR4,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR5
-      COMPAQ_SMART2_MAJOR5,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR6
-      COMPAQ_SMART2_MAJOR6,
-#endif
-#ifdef	COMPAQ_SMART2_MAJOR7
-      COMPAQ_SMART2_MAJOR7,
 #endif
       -1
    };
 
-
-
-#ifdef DEBUG
    debug_enter ( "lvm_check_dev -- CALLED\n");
-#endif
 
-   if ( stat_b == NULL ||
-        ( check_mode != TRUE && check_mode != FALSE) ) ret = -LVM_EPARAM;
-   else {
-      major = MAJOR ( stat_b->st_rdev);
-      for ( i = 0; valid_majors[i] != -1; i++) {
-         if (  stat_b->st_rdev == valid_majors[i] ||
-               major           == valid_majors[i]) {
+   if ( stat_b == NULL || ( check_mode != TRUE && check_mode != FALSE))
+      ret = -LVM_EPARAM;
+   else if ( ( ret = lvm_check_partitioned_dev ( stat_b->st_rdev)) != TRUE) {
+      const int major = MAJOR ( stat_b->st_rdev);
+      dev_t *dev;
+      for ( dev = unpartitioned_majors; *dev != -1; dev++) {
+         if ( major == *dev) {
             ret = TRUE;
             break;
          }
@@ -134,8 +77,6 @@ int lvm_check_dev ( struct stat *stat_b, int check_mode) {
    if ( ret == TRUE && check_mode == TRUE && ! S_ISBLK ( stat_b->st_mode))
       ret = FALSE;
 
-#ifdef DEBUG
    debug_leave ( "lvm_check_dev -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
