@@ -187,28 +187,28 @@ char *lvm_error ( int error) {
       { LVM_EPV_CHECK_CONSISTENCY_ALL_PV_PE,
         "pv_check_consistency_all_pv(): PE"},
       { LVM_EPV_CHECK_CONSISTENCY_LVM_ID,
-        "pv_check_consistencyv(): LVM identifier"},
+        "pv_check_consistency(): LVM identifier"},
       { LVM_EPV_CHECK_CONSISTENCY_LV_CUR,
-        "pv_check_consistencyv(): current LV"},
+        "pv_check_consistency(): current LV"},
       { LVM_EPV_CHECK_CONSISTENCY_MAJOR,
-        "pv_check_consistencyv(): major device number"},
+        "pv_check_consistency(): major device number"},
       { LVM_EPV_CHECK_CONSISTENCY_PE_ALLOCATED,
-        "pv_check_consistencyv(): allocated PE"},
-      { LVM_EPV_CHECK_CONSISTENCY_PE_SIZE, "pv_check_consistencyv(): PE size"},
+        "pv_check_consistency(): allocated PE"},
+      { LVM_EPV_CHECK_CONSISTENCY_PE_SIZE, "pv_check_consistency(): PE size"},
       { LVM_EPV_CHECK_CONSISTENCY_PE_STALE,
-        "pv_check_consistencyv(): stale PE"},
+        "pv_check_consistency(): stale PE"},
       { LVM_EPV_CHECK_CONSISTENCY_PE_TOTAL,
-        "pv_check_consistencyv(): total PE"},
+        "pv_check_consistency(): total PE"},
       { LVM_EPV_CHECK_CONSISTENCY_PV_ALLOCATABLE,
-        "pv_check_consistencyv(): allocatable PE"},
-      { LVM_EPV_CHECK_CONSISTENCY_PV_NAME, "pv_check_consistencyv(): PV name"},
-      { LVM_EPV_CHECK_CONSISTENCY_PV_SIZE, "pv_check_consistencyv(): PV size"},
+        "pv_check_consistency(): allocatable PE"},
+      { LVM_EPV_CHECK_CONSISTENCY_PV_NAME, "pv_check_consistency(): PV name"},
+      { LVM_EPV_CHECK_CONSISTENCY_PV_SIZE, "pv_check_consistency(): PV size"},
       { LVM_EPV_CHECK_CONSISTENCY_PV_STATUS,
-        "pv_check_consistencyv(): PV status"},
+        "pv_check_consistency(): PV status"},
       { LVM_EPV_CHECK_CONSISTENCY_STRUCT_VERSION,
-        "pv_check_consistencyv(): structure version"},
+        "pv_check_consistency(): structure version"},
       { LVM_EPV_CHECK_CONSISTENCY_VG_NAME,
-        "pv_check_consistencyv(): VG name"},
+        "pv_check_consistency(): VG name"},
       { LVM_EPV_CHECK_NAME, "pv_check_name(): PV name"},
       { LVM_EPV_CHECK_NAME_STAT, "pv_check_name(): stat"},
       { LVM_EPV_CHECK_NUMBER_MALLOC, "pv_check_number(): malloc"},
@@ -356,7 +356,7 @@ char *lvm_error ( int error) {
       { LVM_EVG_EXTEND_REDUCE_OPEN, "vg_extend_reduce(): open"},
       { LVM_EVG_READ_LSEEK, "vg_read(): lseek"},
       { LVM_EVG_READ_LVM_STRUCT_VERSION,
-        "vg_read(): LVM structure version nit supported/invalid"},
+        "vg_read(): LVM structure version not supported/invalid"},
       { LVM_EVG_READ_OPEN, "vg_read(): open"},
       { LVM_EVG_READ_PV, "vg_read(): PV"},
       { LVM_EVG_READ_READ, "vg_read(): read"},
@@ -369,7 +369,8 @@ char *lvm_error ( int error) {
         "vg_read_with_pv_and_lv(): current PV"},
       { LVM_EVG_RENAME_OPEN, "vg_rename(): PV open"},
       { LVM_EVG_SETUP_FOR_CREATE_MALLOC, "vg_setup_for_create(): malloc"},
-      { LVM_EVG_SETUP_FOR_CREATE_PV_SIZE, "vg_setup_for_create(): PV size"},
+      { LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MIN, "vg_setup_for_create(): PV"
+        "size too small"},
       { LVM_EVG_SETUP_FOR_CREATE_VG_NUMBER, "vg_setup_for_create(): VG number"},
       { LVM_EVG_SETUP_FOR_EXTEND_MALLOC, "vg_setup_for_extend(): malloc"},
       { LVM_EVG_SETUP_FOR_EXTEND_MAX_PV, "vg_setup_for_extend(): PV maximum"},
@@ -415,13 +416,12 @@ char *lvm_error ( int error) {
       { LVM_EVG_WRITE_LSEEK, "vg_write(): lseek"},
       { LVM_EVG_WRITE_OPEN, "vg_write(): open"},
       { LVM_EVG_WRITE_WRITE, "vg_write(): write"},
+      { LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MAX, "vg_setup_for_create(): PV"
+        "size too large"},
       { 0, NULL}
    };
 
-
-#ifdef DEBUG
    debug_enter ( "lvm_error -- CALLED with: %d\n", error);
-#endif
 
    for ( c = 0; lvm_errors[c].error != 0; c++) {
       if ( lvm_errors[c].error == -error) break;
@@ -431,9 +431,6 @@ char *lvm_error ( int error) {
       ret = strerror ( errno);
    } else ret = lvm_errors[c].errstr;
 
-#ifdef DEBUG
-   debug_enter ( "lvm_error -- LEAVING with: \"%s\"\n", ret);
-#endif
-
+   debug_leave ( "lvm_error -- LEAVING with: \"%s\"\n", ret);
    return ret;
 }
