@@ -1,11 +1,12 @@
 /*
  * tools/lib/vg_check_exist.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March, November 1997
  * February,May,September 1999
  * January 2000
+ * April,May 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -34,6 +35,9 @@
  *    20/05/1999 - fixed NULL pointer bug
  *    29/10/1999 - fixed possible free() bug
  *    08/02/2000 - use debug_enter()/debug_leave()
+ *    09/04/2001 - cleaned up debug output (Andreas Dilger)
+ *    22/05/2001 - fixed pv cache flush related bug causing invalid
+ *                 pointer accesses in vg_check_exist()
  *
  */
 
@@ -46,43 +50,45 @@ int vg_check_exist ( char *vg_name) {
    vg_t *vg = NULL;
    pv_t **pv = NULL;
 
-#ifdef DEBUG
-   debug_enter ( "vg_check_exist -- CALLED with %s\n", vg_name);
-#endif
+   debug_enter ( "vg_check_exist -- CALLED with VG: %s\n", vg_name);
 
    if ( vg_name == NULL || vg_check_name ( vg_name) < 0) ret = -LVM_EPARAM;
-   else if ( ( ret = vg_read ( vg_name, &vg)) == 0) {
-#ifdef DEBUG
+   else {
       debug ( "vg_check_exist -- before pv_read_all_pv_of_vg\n");
-#endif
+      /* reread must be TRUE here to ensure reading the disks after a change */
       if ( ( ret = pv_read_all_pv_of_vg ( vg_name, &pv, TRUE)) == 0) {
-         ret = TRUE;
-         for ( p = 0; pv != NULL && pv[p] != NULL; p++) {
-            if ( system_id_check_exported ( pv[p]->system_id) == TRUE)
-               pv[p]->vg_name[strlen(pv[p]->vg_name)-strlen(EXPORTED)] = 0;
-            if ( strcmp ( vg_name, pv[p]->vg_name) == 0) {
-               pv_count++;
-               if ( pv[p]->version != LVM_STRUCT_VERSION) {
-                  ret = -LVM_EVG_READ_LVM_STRUCT_VERSION;
-                  break;
-               }
-               if ( system_id_check_exported ( pv[p]->system_id) == TRUE) {
-                  ret = -LVM_EPV_READ_PV_EXPORTED;
-                  break;
+         if ( pv != NULL) {
+	    for ( p = 0; pv[p] != NULL; p++) {
+               if ( system_id_check_exported ( pv[p]->system_id) == TRUE)
+                  pv[p]->vg_name[strlen(pv[p]->vg_name)-strlen(EXPORTED)] = 0;
+               if ( strcmp ( vg_name, pv[p]->vg_name) == 0) {
+                  pv_count++;
+                  if ( pv[p]->version != LVM_STRUCT_VERSION) {
+                     ret = -LVM_EVG_READ_LVM_STRUCT_VERSION;
+                     break;
+                  }
+                  if ( system_id_check_exported ( pv[p]->system_id) == TRUE) {
+                     ret = -LVM_EPV_READ_PV_EXPORTED;
+                     break;
+                  }
                }
             }
          }
       }
-#ifdef DEBUG
-      debug ( "vg_check_exist -- before vg->pv_cur check with "
-               "vg->pv_cur: %lu  pv_count: %d\n", vg->pv_cur, pv_count);
-#endif
-      if ( vg->pv_cur != pv_count) ret = -LVM_EVG_CHECK_EXIST_PV_COUNT;
    }
 
-#ifdef DEBUG
+   if ( ret == 0) {
+      if ( ( ret = vg_read ( vg_name, &vg)) == 0) {
+         debug ( "vg_check_exist -- before vg->pv_cur check with "
+                  "vg->pv_cur: %lu  pv_count: %d\n", vg->pv_cur, pv_count);
+         if ( vg->pv_cur != pv_count)
+            ret = -LVM_EVG_CHECK_EXIST_PV_COUNT;
+	 else
+            ret = TRUE;
+      }
+   }
+
    debug_leave ( "vg_check_exist -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
@@ -96,9 +102,7 @@ char **vg_check_exist_all_vg ( void) {
    static char **vg_name_ptr = NULL;
    pv_t **pv = NULL;
 
-#ifdef DEBUG
    debug_enter ( "vg_check_exist_all_vg -- CALLED\n");
-#endif
 
    if ( vg_name_ptr != NULL) {
       for ( v = 0; vg_name_ptr[v] != NULL; v++) free ( vg_name_ptr[v]);
@@ -114,23 +118,16 @@ char **vg_check_exist_all_vg ( void) {
    nv = 0;
    for ( p = 0; pv[p] != NULL; p++) {
       if ( pv[p] != NULL && pv[p]->vg_name[0] != 0) {
-#ifdef DEBUG
          debug ( "vg_check_exist_all_vg -- checking vg_name: %s[%d/%d]\n",
                  pv[p]->vg_name, p, nv);
-#endif
          for ( v = 0; v < nv && vg_name_ptr[v] != NULL; v++)
             if ( strcmp ( pv[p]->vg_name, vg_name_ptr[v]) == 0) break;
          if ( v < nv) continue;
-
-#ifdef DEBUG
             debug ( "vg_check_exist_all_vg -- before vg_check_exist ( %s)\n",
                     pv[p]->vg_name);
-#endif
          if ( ( ret = vg_check_exist ( pv[p]->vg_name)) == TRUE) {
-#ifdef DEBUG
             debug ( "vg_check_exist_all_vg -- HIT vg_name: %s\n",
                     pv[p]->vg_name);
-#endif
             vg_name_ptr_sav = vg_name_ptr;
             if ( ( vg_name_ptr =
                       realloc ( vg_name_ptr,
@@ -156,18 +153,15 @@ char **vg_check_exist_all_vg ( void) {
                goto vg_check_exist_all_vg_end;
             }
             strcpy ( vg_name_ptr[nv], pv[p]->vg_name);
-#ifdef DEBUG
             debug ( "vg_check_exist_all_vg -- vg_name_ptr[%d]: %s\n",
                      nv, vg_name_ptr[nv]);
-#endif
             nv++;
          }
       }
    }
 
 vg_check_exist_all_vg_end:
-#ifdef DEBUG
-   debug_leave ( "vg_check_exist_all_vg -- LEAVING with %s\n", vg_name_ptr);
-#endif
+
+   debug_leave ( "vg_check_exist_all_vg -- LEAVING with %d VGs\n", nv);
    return vg_name_ptr;
 }
