@@ -33,10 +33,7 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
    pv_t pv_this;
    pv_t *pv_this_ptr = &pv_this;
 
-
-#ifdef DEBUG
    debug_enter ( "vg_read_from_pv -- CALLED\n");
-#endif
 
    if ( pv_name == NULL ||
         pv_check_name ( pv_name) < 0 ||
@@ -45,7 +42,9 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
       goto vg_read_from_pv_end;
    }
 
-   if ( pv_read ( pv_name, &pv_this_ptr, NULL) == 0) {
+   if ( ( ret = pv_read ( pv_name, &pv_this_ptr, NULL)) == 0 ||
+	   ret == -LVM_EPV_READ_MD_DEVICE ||
+	   ret == -LVM_EPV_READ_PV_EXPORTED) {
       /* read the VG info */
       if ( ( pv_handle = open ( pv_name, O_RDONLY)) == -1)
          ret = -LVM_EVG_READ_OPEN;
@@ -53,6 +52,7 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
                 pv_this_ptr->vg_on_disk.base) ret = -LVM_EVG_READ_LSEEK;
       else if ( read ( pv_handle, &vg_this, sizeof ( vg_this)) != \
                 sizeof ( vg_this)) ret = -LVM_EVG_READ_READ;
+      else ret = 0;
       if ( pv_handle != -1) close ( pv_handle);
    
       if ( ret == 0) {
@@ -69,8 +69,7 @@ int vg_read_from_pv ( char *pv_name, vg_t **vg) {
    if ( ret != 0 && ret != -LVM_EVG_READ_VG_EXPORTED) *vg = NULL;
 
 vg_read_from_pv_end:
-#ifdef DEBUG
+
    debug_leave ( "vg_read_from_pv -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
