@@ -1,11 +1,12 @@
 /*
  * tools/lvmsar.c
  *
- * Copyright (C) 1998  Heinz Mauelshagen, Germany
+ * Copyright (C) 1998  Heinz Mauelshagen, Sistina Software
  *
  * September 1998
  * October 1999
  * February 2000
+ * March 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +30,8 @@
  *
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    13/03/2001 - added support to read lvmsadc(8) created data from <stdin>
  *
  */
 
@@ -44,23 +47,19 @@ int main ( int argc, char **argv) {
    int flag = 0;
    int le = 0;
    int opt_f = 0;
+   int opt_s = 0;
    int opt_v = 0;
    int read_sum = 0;
    int reads = 0;
    int writes = 0;
    int write_sum = 0;
    char buffer[80];
-#ifdef DEBUG
-   char *options = "dfh?v";
-#else
-   char *options = "fh?v";
-#endif
+   char *options = "fh?sv" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
-#ifdef DEBUG
-      { "debug",   no_argument, NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "full",    no_argument, NULL, 'f'},
       { "help",    no_argument, NULL, 'h'},
+      { "stdin",   no_argument, NULL, 's'},
       { "verbose", no_argument, NULL, 'v'},
       { NULL, 0, NULL, 0}
    };
@@ -68,22 +67,13 @@ int main ( int argc, char **argv) {
    char *ptr = NULL;
    FILE *log = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
-
    while ( ( c = getopt_long ( argc, argv, options,
                                long_options,  NULL)) != EOF) {
       switch ( c) {
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'f':
             if ( opt_f > 0) {
@@ -99,17 +89,24 @@ int main ( int argc, char **argv) {
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
+                     DEBUG_HELP
                      "\t[-f/--full]\n"
                      "\t[-h/-?/--help]\n"
+                     "\t[-s/--stdin]\n"
                      "\t[-v/--verbose]\n"
                      "\tLogFilePath\n",
                      lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
+         case 's':
+            if ( opt_s > 0) {
+               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            opt_s++;
+            break;
+
          case 'v':
             if ( opt_v > 0) {
                fprintf ( stderr, "%s -- v option already given\n\n", cmd);
@@ -127,17 +124,22 @@ int main ( int argc, char **argv) {
 
    CMD_MINUS_CHK;
 
-   if ( argc - optind != 1) {
-      fprintf ( stderr, "%s -- please enter a LogFilePath\n\n", cmd);
+   if ( opt_s == 0 && argc - optind != 1) {
+      fprintf ( stderr, "%s -- please enter a LogFilePath or use option -s to read from <stdin>\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
-   logfile = argv[optind];
 
-   if ( opt_v > 0) printf ( "%s -- opening logfile \"%s\"\n", cmd, logfile);
-   if ( ( log = fopen ( logfile, "r")) == NULL) {
-      fprintf ( stderr, "%s -- ERROR \"%s\" opening \"%s\" for read\n\n",
-                        cmd, lvm_error ( errno), logfile);
-      return LVM_ELVMSAR_FOPEN;
+   if ( opt_s == 0) {
+      logfile = argv[optind];
+      if ( opt_v > 0) printf ( "%s -- opening logfile \"%s\"\n", cmd, logfile);
+      if ( ( log = fopen ( logfile, "r")) == NULL) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" opening \"%s\" for read\n\n",
+                           cmd, lvm_error ( errno), logfile);
+         return LVM_ELVMSAR_FOPEN;
+      }
+   } else {
+      logfile = "<stdin>";
+      log = stdin;
    }
 
    if ( opt_v > 0) printf ( "%s -- reading logfile \"%s\"\n", cmd, logfile);
@@ -179,11 +181,13 @@ int main ( int argc, char **argv) {
    if ( flag > 0) printf ( "Total reads: %8d   Total writes: %8d\n\n",
                            read_sum, write_sum);
 
-   if ( opt_v > 0) printf ( "%s -- closing logfile \"%s\"\n", cmd, logfile);
-   if ( fclose ( log) < 0) {
-      fprintf ( stderr, "%s -- ERROR \"%s\" closing \"%s\"\n\n",
-                        cmd, lvm_error ( errno), logfile);
-      return LVM_ELVMSAR_FCLOSE;
+   if ( log != stdin) {
+      if ( opt_v > 0) printf ( "%s -- closing logfile \"%s\"\n", cmd, logfile);
+      if ( fclose ( log) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" closing \"%s\"\n\n",
+                           cmd, lvm_error ( errno), logfile);
+         return LVM_ELVMSAR_FCLOSE;
+      }
    }
 
    return 0;
