@@ -1,11 +1,13 @@
 /*
  * tools/lvreduce.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * April 1997
  * April-June,September 1998
- * February,July 1999
+ * February,July,August,October 1999
+ * February,November 2000
+ * January,July 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +39,12 @@
  *    04/09/1998 - corrected some messages
  *    06/02/1999 - fixed lvm_check_number() usage
  *    06/07/1999 - avoided vg_status()
+ *    15/08/1999 - enhanced to avoid reduction of snapshot logical volumes
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    07/11/2000 - used LVM_VG_NAME environment variable
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    25/07/2001 - avoid misleading "(null)" in error message (AGK)
  *
  */
 
@@ -53,6 +61,7 @@ int main ( int argc, char **argv) {
    int c1 = 0;
    int l = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_f = 0;
    int opt_l = 0;
    int opt_L = 0;
@@ -60,26 +69,36 @@ int main ( int argc, char **argv) {
    int ret = 0;
    int sign = 0;
    int size_rest = 0;
-   uint new_size = 0;
+   long new_size = 0;
+   char buffer[NAME_LEN];
    char *dummy = NULL;
-#ifdef DEBUG
-   char *options = "A:dfh?l:L:yv";
-#else
-   char *options = "A:fh?l:L:yv";
-#endif
+   char *options = "A:fh?l:L:yv" DEBUG_SHORT_OPTION;
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+      DEBUG_LONG_OPTION
+      { "force",      no_argument,       NULL, 'f'},
+      { "help",       no_argument,       NULL, 'h'},
+      { "extents",    required_argument, NULL, 'l'},
+      { "size",       required_argument, NULL, 'L'},
+      { "yes",        no_argument,       NULL, 'v'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL,         0,                 NULL, 0}
+   };
    char *lv_name = NULL;
    char *vg_name = NULL;
    vg_t *vg = NULL;
    lv_t *lv = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -93,15 +112,7 @@ int main ( int argc, char **argv) {
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'f':
             if ( opt_f > 0) {
@@ -113,21 +124,19 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Reduce\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Reduce\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-f]\n"
-                     "\t[-h/?]\n"
-                     "\t{-l [-]LogicalExtentsNumber |\n"
-                     "\t -L [-]LogicalVolumeSize[kKmMgGtT]}\n"
-                     "\t[-v]\n"
-                     "\tLogicalVolumePath\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-A/--autobackup y/n]\n"
+                     DEBUG_HELP
+                     "\t[-f/--force]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t{-l/--extents [-]LogicalExtentsNumber |\n"
+                     "\t -L/--size [-]LogicalVolumeSize[kKmMgGtT]}\n"
+                     "\t[-v/--verbose]\n"
+                     "\tLogicalVolume[Path]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -183,12 +192,21 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK
+   CMD_CHECK_OPT_A_SET;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a logical volume name\n\n", cmd);
       return LVM_ELVREDUCE_LV_MISSING;
    }
    lv_name = argv[optind];
+   LVM_CHECK_DEFAULT_VG_NAME ( lv_name, buffer, sizeof ( buffer));
+
+   if ( lv_name == NULL) {
+      fprintf ( stderr, "%s -- a path needs supplying for "
+                        "logical volume argument \"%s\"\n\n",
+                        cmd, argv[optind]);
+      return LVM_ELVREDUCE_LV_NAME;
+   }
 
    if ( lv_check_name ( lv_name) < 0) {
       fprintf ( stderr, "%s -- no valid logical volume name \"%s\"\n\n",
@@ -238,6 +256,17 @@ int main ( int argc, char **argv) {
       return LVM_ELVREDUCE_LV_GET_INDEX;
    }
 
+   if ( opt_v > 0) printf ( "%s -- getting status of logical volume \"%s\" "
+                            "from kernel\n",
+                            cmd, lv_name);
+
+   if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
+                        "volume \"%s\"\n\n",
+                        cmd, lvm_error ( ret), lv_name);
+      return LVM_ELVREDUCE_LV_STATUS_BYNAME;
+   }
+
    if ( opt_L > 0 || opt_l > 0) {
       if ( opt_L > 0) new_size *= 2;
       else            new_size *= vg->pe_size;
@@ -255,25 +284,27 @@ int main ( int argc, char **argv) {
       size_rest = new_size %
                   ( vg->lv[l]->lv_stripes * vg->pe_size);
       if ( size_rest != 0) {
-         printf ( "%s -- rounding size %d KB to stripe boundary size ",
+         printf ( "%s -- rounding size %ld KB to stripe boundary size ",
                   cmd, new_size / 2);
          new_size = new_size - size_rest;
-         printf ( "%d KB\n", new_size / 2);
+         printf ( "%ld KB\n", new_size / 2);
       }
    } else {
       fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
 
-   if ( opt_v > 0) printf ( "%s -- getting status of logical volume \"%s\" "
-                            "from kernel\n",
-                            cmd, lv_name);
-   if ( lv_status_byname ( vg_name, lv_name, &lv) == 0 &&
-        lv->lv_status == LV_ACTIVE) {
+   if ( new_size <= lv->lv_remap_ptr * lv->lv_chunk_size) {
+      fprintf ( stderr, "%s -- size of snapshot would be too small\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
+
+   if ( lv->lv_status == LV_ACTIVE &&
+        !(lv->lv_access & LV_SNAPSHOT)) {
       printf ( "%s -- WARNING: reducing active", cmd);
       if ( lv->lv_open > 0) printf ( " and open");
       printf ( " logical volume to %s\n"
-               "%s -- THIS MAY DESTROY YOUR DATA\n",
+               "%s -- THIS MAY DESTROY YOUR DATA (filesystem etc.)\n",
                ( dummy = lvm_show_size ( new_size / 2, SHORT)), cmd);
       free ( dummy); dummy = NULL;
    }
@@ -288,7 +319,21 @@ int main ( int argc, char **argv) {
       return LVM_ELVREDUCE_LV_SETUP;
    }
 
-   if ( opt_f == 0) {
+   if ( lv->lv_access & LV_SNAPSHOT) {
+      if ( ( ret = lv_setup_COW_table_for_create (
+                      vg,
+                      vg->lv[lv_get_index_by_minor ( vg, vg->lv[l]->lv_snapshot_minor)]->lv_name,
+                      l,
+                      lv->lv_chunk_size * SECTOR_SIZE / 1024)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot "
+                           "copy on write exception table for \"%s\"\n\n",
+                           cmd, lvm_error ( ret), lv_name);
+         return LVM_ELVEXTEND_LV_SETUP_COW_TABLE_FOR_CREATE;
+      }
+   }
+
+   if ( opt_f == 0 &&
+        !(lv->lv_access & LV_SNAPSHOT)) {
       c = 0;
       while ( c != 'n' && c != 'y') {
          if ( c == '\n' || c == 0)
@@ -311,18 +356,18 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- creating reduced logical volume "
                             "VGDA in kernel\n", cmd);
    if ( ( ret = lv_reduce ( vg, vg->lv[l], lv_name)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d reducing logical volume \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" reducing logical volume \"%s\" "
                         "in kernel\n\n", 
-                        cmd, ret, lv_name);
+                        cmd, lvm_error ( ret), lv_name);
       return LVM_ELVREDUCE_LV_REDUCE;
    }
 
    if ( opt_v > 0) printf ( "%s -- storing reduced logical volume "
                             "data on disk(s)\n", cmd);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: storing data of volume group \"%s\" "
-                        "on disks\n\n", 
-                        cmd, ret, vg_name);
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume "
+                        "group \"%s\" on disks\n\n", 
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_ELVREDUCE_VG_WRITE;
    }
 
@@ -333,8 +378,10 @@ int main ( int argc, char **argv) {
       printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name);
    }
-
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
