@@ -1,12 +1,13 @@
 /*
  * tools/pvmove.c
  *
- * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * June,October 1997
  * April,May,July,September 1998
  * January,October 1999
  * February 2000
+ * January,February 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +38,13 @@
  *    26/01/1999 - made volume group directory prefix a preprocessor option
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    14/02/2001 - Tidied usage message and options
+ *    19/02/2001 - changed messages to give better understanding of pvmove's
+ *                 behaviour
+ *    20/02/2001 - fixed bug preventing move from/to MD devices by checking
+ *                 LVM_EPV_READ_MD_DEVICE pv_read() return code (Andreas Dilger)
+ *    19/03/2001 - Fix some typos in messages
  *
  */
 
@@ -48,8 +56,33 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
+void usage(int ret)
+{
+   FILE *stream = stderr;
+
+   if (ret == 0) {
+      stream = stdout;
+      printf("%s  (IOP %d)\n\n", lvm_version, LVM_LIB_IOP_VERSION);
+      printf("%s - move extents from physical volume to another\n", cmd);
+   }
+
+   fprintf(stream, "\n%s "
+           "[-A|--autobackup {y|n}] "
+           DEBUG_HELP_2
+           "[-f|--force]"
+           "[-h|--help]\n\t"
+           "[-t|--test] "
+           "[-v[v]|--verbose [--verbose]] "
+           "[--version]\n\t"
+           "[{-n|--name} LogicalVolume[:LogicalExtent[-LogicalExtent]...]]\n\t"
+           "SourcePhysicalVolume[:PhysicalExtent[-PhysicalExtent]...]}\n\t"
+           "[DestinationPhysicalVolume[:PhysicalExtent[-PhysicalExtent]...]...]\n\n",
+           cmd);
+   exit(ret);
+}
 
 int main ( int argc, char **argv) {
+   int buffer_size = 64*1024;
    int c = 0;
    int c1 = 0;
    int ext = 0;
@@ -75,21 +108,16 @@ int main ( int argc, char **argv) {
    char lv_name_buf[NAME_LEN] = { 0, };
    char *buffer = NULL;
    char *lv_name = NULL;
-#ifdef DEBUG
-   char *options = "A:dfh?n:tv";
-#else
-   char *options = "A:fh?n:tv";
-#endif
+   char *options = "A:fhn:tv" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
       { "autobackup", required_argument, NULL, 'A'},
-#ifdef DEBUG
-      { "debug",      no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "force",      no_argument,       NULL, 'f'},
       { "help",       no_argument,       NULL, 'h'},
       { "name",       required_argument, NULL, 'n'},
       { "test",       no_argument,       NULL, 't'},
       { "verbose",    no_argument,       NULL, 'v'},
+      { "version",    no_argument,       NULL, 22 },
       { NULL, 0, NULL, 0}
    };
    char **pv_allowed = NULL;
@@ -99,9 +127,9 @@ int main ( int argc, char **argv) {
    pv_t *pv = NULL;
    vg_t *vg = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
@@ -109,105 +137,63 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             opt_A_set++;
-            if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             if ( strcmp ( optarg, "y") == 0);
             else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
             else {
-               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
+               fprintf ( stderr, "%s -- invalid autobackup argument '%s'\n",
                                  cmd, optarg);
-               return LVM_EINVALID_CMD_LINE;
+               usage(LVM_EINVALID_CMD_LINE);
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'f':
-            if ( opt_f > 0) {
-               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_f++;
             break;
 
          case 'h':
-         case '?':
-            printf ( "\n%s  (IOP %d)\n\n%s -- Physical Volume Move\n\n"
-                     "Synopsis:\n"
-                     "---------\n\n"
-                     "%s\n"
-                     "\t[-A/--autobackup y/n]\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
-                     "\t[-f/--force]\n"
-                     "\t[-h/-?/--help]\n"
-                     "\t[-n/--name LogicalVolumePath[:LogicalExtend"
-                     "[-LogicalExtend]...]] /\n"
-                     "\tSourcePhysicalVolumePath[:PhysicalExtend"
-                     "[-PhysicalExtend]...]\n"
-                     "\t[-t/--test]\n"
-                     "\t[-v[v]/--verbose [--verbose]]\n"
-                     "\t[DestinationPhysicalVolumePath"
-                     "[:PhysicalExtend[-PhysicalExtend]...]...]"
-                     "\n\n",
-                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
-            return 0;
+            usage(0);
             break;
 
          case 'n':
-            if ( opt_n > 0) {
-               fprintf ( stderr, "%s -- n option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             lv_name = optarg;
-            if ( lvm_get_col_numbers ( optarg, &les) < 0) {
-               return LVM_EPVMOVE_LVM_GET_COL_NUMBERS;
-            }
+            if ( lvm_get_col_numbers ( optarg, &les) < 0)
+               usage(LVM_EPVMOVE_LVM_GET_COL_NUMBERS);
             opt_n++;
             break;
 
          case 't':
-            if ( opt_t > 0) {
-               fprintf ( stderr, "%s -- t option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_t++;
             break;
 
          case 'v':
-            if ( opt_v > 1) {
-               fprintf ( stderr, "%s -- v option already given two times\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_v++;
             break;
 
+         case 22:
+            printf ("%s: %s (IOP %d)\n", cmd, lvm_version, LVM_LIB_IOP_VERSION);
+            return 0;
+
          default:
-            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
-                      cmd, c);
-            return LVM_EINVALID_CMD_LINE;
+            fprintf(stderr, "%s -- invalid option '%c'\n", cmd, c);
+         case '?':
+            usage(LVM_EINVALID_CMD_LINE);
       }
    }
 
+   /* Not until all commands are converted.
    CMD_MINUS_CHK;
+   */
+   if ( optind < argc && *argv[optind] == '-') {
+      fprintf(stderr, "%s -- invalid option '%s'\n", argv[optind], cmd);
+      usage(LVM_EINVALID_CMD_LINE);
+   }
    CMD_CHECK_OPT_A_SET;
 
-
    if ( opt_n == 0 && optind == argc) {
-      fprintf ( stderr, "%s -- please enter a source physical volume path\n\n",
-                        cmd);
-      return LVM_EPVMOVE_PV_SOURCE;
+      fprintf(stderr, "%s -- no source physical volume path given\n\n", cmd);
+      usage(LVM_EPVMOVE_PV_SOURCE);
    }
 
    src_pv_name = argv[optind++];
@@ -216,23 +202,23 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- ERROR: invalid physical extent numbers "
                         "on command line\n",
                         cmd);
-      return LVM_EPVMOVE_LVM_GET_COL_NUMBERS;
+      usage(LVM_EPVMOVE_LVM_GET_COL_NUMBERS);
    }
 
    if ( opt_n > 0 && source_pes != NULL && source_pes[0] != -1) {
       fprintf ( stderr, "%s -- ERROR: you can't select a logical volume AND "
                         "source physical extents\n\n",
                         cmd);
-      return LVM_EINVALID_CMD_LINE;
+      usage(LVM_EINVALID_CMD_LINE);
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking name of source physical "
-                            "volume \"%s\"\n",
-                            cmd, src_pv_name);
+   if ( opt_v > 0)
+      printf("%s -- checking name of source physical volume \"%s\"\n",
+             cmd, src_pv_name);
    if ( pv_check_name ( src_pv_name) < 0) {
       fprintf ( stderr, "%s -- \"%s\" is an invalid physical volume name\n\n",
                         cmd, src_pv_name);
-      return LVM_EPVMOVE_PV_CHECK_NAME_SOURCE;
+      usage(LVM_EPVMOVE_PV_CHECK_NAME_SOURCE);
    }
 
    LVM_CHECK_IOP;
@@ -241,7 +227,8 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0) printf ( "%s -- reading data of source physical volume "
                             "from \"%s\"\n", cmd, src_pv_name);
-   if ( ( ret = pv_read ( src_pv_name, &pv, NULL)) < 0) {
+   if ( ( ret = pv_read ( src_pv_name, &pv, NULL)) < 0 &&
+        ret != -LVM_EPV_READ_MD_DEVICE) {
       if ( ret == -LVM_EPV_READ_OPEN) {
          fprintf ( stderr, "%s -- source physical volume \"%s\" "
                            "doesn't exist\n\n",
@@ -314,7 +301,7 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\"\n",
                                cmd, lv_name);
       if ( strchr ( lv_name, '/') == NULL) {
-         sprintf ( lv_name_buf, LVM_DIR_PREFIX "%s/%s%c", vg_name, lv_name, 0);
+         sprintf ( lv_name_buf, LVM_DIR_PREFIX "%s/%s", vg_name, lv_name);
          lv_name = lv_name_buf;
       }
       if ( lv_check_name ( lv_name) < 0) {
@@ -362,7 +349,8 @@ int main ( int argc, char **argv) {
                               cmd, argv[i]);
             return LVM_EPVMOVE_PV_CHECK_NAME;
          }
-         if ( ( ret = pv_read ( argv[i], &pv, NULL)) < 0) {
+         if ( ( ret = pv_read ( argv[i], &pv, NULL)) < 0 &&
+              ret != -LVM_EPV_READ_MD_DEVICE) {
             if ( ret == -LVM_EPV_READ_OPEN) {
                fprintf ( stderr, "%s -- ERROR: destination physical volume "
                                  "%s doesn't exist\n\n",
@@ -466,7 +454,7 @@ int main ( int argc, char **argv) {
    }
 
    /* allocate transfer buffer memory */
-   if ( ( buffer = malloc ( vg->pe_size * SECTOR_SIZE)) == NULL) {
+   if ( ( buffer = malloc ( buffer_size)) == NULL) {
       fprintf ( stderr, "%s -- malloc error in file \"%s\" [line: %d]\n\n",
                         cmd, __FILE__, __LINE__);
       return LVM_EPVMOVE_MALLOC;
@@ -483,8 +471,8 @@ int main ( int argc, char **argv) {
    printf ( "active volume group \"%s\"\n", vg_name);
 
    if ( opt_f == 0) {
-      printf ( "%s -- WARNING: moving of active logical volumes "
-               "may cause data loss!\n", cmd);
+      printf ( "%s -- WARNING: if you lose power during the move you may need\n"
+               "\tto restore your LVM metadata from backup!\n", cmd);
       c = 0;
       while ( c != 'y' && c != 'n') {
          if ( c == '\n' || c == 0)
@@ -512,7 +500,7 @@ int main ( int argc, char **argv) {
    pe_moved = 0;
    ret = 1;
    while ( ret > 0) {
-      if ( ( ret = pv_move_pes ( vg, buffer, pv_allowed,
+      if ( ( ret = pv_move_pes ( vg, buffer, buffer_size, pv_allowed,
                                  src_pv_index, lv_index,
                                  les, source_pes, dest_pes,
                                  opt_v, opt_t)) < 0) {
@@ -553,7 +541,7 @@ int main ( int argc, char **argv) {
          vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
       } else {
          printf ( "%s -- WARNING: you don't have an automatic "
-                  "backup of \"%s\"\n",
+                  "metadata backup of \"%s\"\n",
                   cmd, vg_name);
       }
    }
