@@ -1,27 +1,28 @@
 /*
  * tools/vgcreate.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March-June 1997
  * May,June,September 1998
  * January,February,October 1999
  * February 2000
+ * January,June 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * LVM is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA. 
+ * Boston, MA 02111-1307, USA.
  *
  */
 
@@ -44,6 +45,14 @@
  *    06/02/1999 - fixed lvm_check_number() usage
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    22/01/2001 - fixed PE size default unit bug
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    29/01/2001 - changed type of pe_size var for correct use
+ *                 with lvm_check_number() I forgot last time
+ *    19/03/2001 - Improved the phrasing of error message
+ *    09/04/2001 - Tidy usage message and command-line options.
+ *    22/06/2001 - added Andreas Dilger's PE on 4k boundary alignment enhancements
+ *    27/06/2001 - fixed minimum PV size message
  *
  */
 
@@ -55,6 +64,31 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
+void usage(int ret)
+{
+   FILE *stream = stderr;
+
+   if (ret == 0) {
+      printf("%s (IOP %d)\n\n", lvm_version, LVM_LIB_IOP_VERSION);
+      printf("%s -- initialize a volume group for use by LVM\n", cmd);
+      stream = stdout;
+   }
+
+   fprintf(stream, "\n%s "
+           "[-A|--autobackup {y|n}] "
+           DEBUG_HELP_2
+           "[-l|--maxlogicalvolumes MaxLogicalVolumes]\n\t"
+           "[-p|--maxphysicalvolumes MaxPhysicalVolumes] "
+           "[-h|--help]\n\t"
+           "[-s|--physicalextentsize PhysicalExtentSize[kKmMgGtT]] "
+           "[-v|--verbose]\n\t"
+           "[--version] "
+           "VolumeGroupName "
+           "PhysicalVolume [PhysicalVolume...]\n",
+           cmd);
+   exit ( ret);
+}
+
 int main ( int argc, char **argv) {
    int c = 0;
    int count_sav = 0;
@@ -67,12 +101,12 @@ int main ( int argc, char **argv) {
    int opt_v = 0;
    int p = 0;
    int p1 = 0;
-   int max_lv = MAX_LV;
-   int max_pv = MAX_PV;
+   int max_lv = MAX_LV - 1;
+   int max_pv = MAX_PV - 1;
    int min_pv_index = 0;
    ulong max_pv_size = 0;
    ulong min_pv_size = -1;
-   ulong pe_size = LVM_DEFAULT_PE_SIZE;
+   long pe_size = LVM_DEFAULT_PE_SIZE;
    int ret = 0;
    int size = 0;
    int v = 0;
@@ -82,29 +116,24 @@ int main ( int argc, char **argv) {
    vg_t vg;
    pv_t **pv = NULL;
    pv_t **pvp = NULL;
-#ifdef DEBUG
-   char *options = "A:dh?l:p:s:v";
-#else
-   char *options = "A:h?l:p:s:v";
-#endif
+   char *options = "A:h?l:p:s:v" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
       { "autobackup",         required_argument, NULL, 'A'},
-#ifdef DEBUG
-      { "debug",              no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "help",               no_argument,       NULL, 'h'},
       { "maxlogicalvolumes",  required_argument, NULL, 'l'},
       { "maxphysicalvolumes", required_argument, NULL, 'p'},
       { "physicalextentsize", required_argument, NULL, 's'},
       { "verbose",            no_argument,       NULL, 'v'},
+      { "version",            no_argument,       NULL, 22},
       { NULL, 0, NULL, 0}
    };
    char *pv_name = NULL;
    char **vg_name_ptr = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
@@ -112,134 +141,103 @@ int main ( int argc, char **argv) {
       switch ( c) {
          case 'A':
             opt_A_set++;
-            if ( opt_A > 1) {
-               fprintf ( stderr, "%s -- A option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            if ( strcmp ( optarg, "y") == 0);
-            else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
-            else {
-               fprintf ( stderr, "%s -- invalid option argument \"%s\"\n\n",
+            opt_A++;
+            if ( optarg[0] == 'n') opt_A = 0;
+            else if ( optarg[0] != 'y') {
+               fprintf ( stderr, "%s -- ERROR option A argument \"%s\"\n",
                                  cmd, optarg);
-               return LVM_EINVALID_CMD_LINE;
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
-         case '?':
-            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Create\n\n"
-                     "Synopsis:\n"
-                     "---------\n\n"
-                     "%s\n"
-                     "\t[-A y/n]\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-l/--maxlogicalvolumes MaxLogicalVolumes]\n"
-                     "\t[-p/--maxphysicalvolumes MaxPhysicalVolumes]\n"
-                     "\t[-h/-?/--help]\n"
-                     "\t[-s/--physicalextentsize "
-                     "PhysicalExtentSize[kKmMgGtT]]\n"
-                     "\t[-v/--verbose]\n"
-                     "\tVolumeGroupName\n"
-                     "\tPhysicalDevicePath [PhysicalDevicePath...]\n\n",
-                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
-            return 0;
+            usage (0);
             break;
 
          case 'l':
-            if ( opt_l > 0) {
-               fprintf ( stderr, "%s -- l option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             if ( ( max_lv = lvm_check_number ( optarg, FALSE)) > 0) {
                opt_l++;
             } else {
-               fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
+               fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n",
                                  cmd, optarg);
-               return LVM_EINVALID_CMD_LINE;
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             break;
 
          case 'p':
-            if ( opt_p > 0) {
-               fprintf ( stderr, "%s -- p option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             if ( ( max_pv = lvm_check_number ( optarg, FALSE)) > 0) {
                opt_p++;
             } else {
-               fprintf ( stderr, "%s -- ERROR option p argument \"%s\"\n\n",
+               fprintf ( stderr, "%s -- ERROR option p argument \"%s\"\n",
                                  cmd, optarg);
-               return LVM_EINVALID_CMD_LINE;
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             break;
 
          case 's':
-            if ( opt_s > 0) {
-               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             if ( ( pe_size = lvm_check_number ( optarg, TRUE)) > 0) {
-               pe_size = ( pe_size * 1024) / SECTOR_SIZE;
-               /* if no unit has been given --> asume megabyte */
-               if ( isdigit ( optarg[strlen ( optarg)-1])) pe_size *= 1024;
+               pe_size = ( (unsigned long long) pe_size * 1024) / SECTOR_SIZE;
                opt_s++;
-               if ( vg_check_pe_size ( pe_size) < 0) {
-                  fprintf ( stderr, "%s -- invalid physical extent size %lu\n"
-                                    "%s -- must be power of 2 and between "
-                                    "%lu and %lu\n\n",
-                                    cmd, pe_size / 2, cmd,
-                                    LVM_MIN_PE_SIZE, LVM_MAX_PE_SIZE);
-                  return LVM_EINVALID_CMD_LINE;
+               if (vg_check_pe_size(pe_size) < 0) {
+		       fprintf(stderr,
+			       "%s -- invalid physical extent size %s\n"
+			       "%s -- must be power of 2 and between "
+			       "%s and %s\n\n",
+			       cmd,
+			       lvm_show_size (sectors_to_k(pe_size), SHORT),
+			       cmd,
+			       lvm_show_size (sectors_to_k(LVM_MIN_PE_SIZE),
+			                      SHORT),
+			       lvm_show_size (sectors_to_k(LVM_MAX_PE_SIZE),
+			                     SHORT));
+		       usage(LVM_EINVALID_CMD_LINE);
                }
             } else {
-               fprintf ( stderr, "%s -- ERROR option s argument \"%s\"\n\n",
+               fprintf ( stderr, "%s -- ERROR option s argument \"%s\"\n",
                                  cmd, optarg);
-               return LVM_EINVALID_CMD_LINE;
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             break;
- 
+
          case 'v':
-            if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_v++;
             break;
 
+         case 22:
+            printf ("%s: %s (IOP %d)\n", cmd, lvm_version, LVM_LIB_IOP_VERSION);
+            return 0;
+
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return LVM_EINVALID_CMD_LINE;
+         case '?':
+            usage ( LVM_EINVALID_CMD_LINE);
       }
    }
 
+   /* Not until all commands are converted
    CMD_MINUS_CHK;
+   */
+   if ( optind < argc && *argv[optind] == '-') {
+      fprintf ( stderr, "%s -- invalid command line\n", cmd);
+      usage ( LVM_EINVALID_CMD_LINE);
+   }
    CMD_CHECK_OPT_A_SET;
 
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
-                        " and physical volumes\n\n", cmd);
-      return LVM_EVGCREATE_VG_AND_PV_MISSING;
+                        " and physical volumes\n", cmd);
+      usage ( LVM_EVGCREATE_VG_AND_PV_MISSING);
    }
    vg_name = argv[optind];
    optind++;
 
    if ( optind == argc) {
-      fprintf ( stderr, "%s -- please enter physical volume names\n\n", cmd);
-      return LVM_EVGCREATE_PV_MISSING;
+      fprintf ( stderr, "%s -- please enter physical volume name(s)\n", cmd);
+      usage ( LVM_EVGCREATE_PV_MISSING);
    }
 
    /* valid VG name? */
@@ -247,18 +245,18 @@ int main ( int argc, char **argv) {
    if ( ( ret = vg_check_name ( vg_name)) < 0) {
       fprintf ( stderr, "%s -- invalid volume group name \"%s\"\n\n",
                 cmd, vg_name);
-      return LVM_EVGCREATE_VG_CHECK_NAME;
-   } 
+      usage ( LVM_EVGCREATE_VG_CHECK_NAME);
+   }
 
    /* does /dev/VolumeGroupName already exist? */
-   if ( opt_v > 0) printf ( "%s -- checking volume group "
-                            "directory existence\n", cmd);
+   if ( opt_v > 0)
+      printf ( "%s -- checking volume group directory existence\n", cmd);
    if ( ( ret = vg_check_dir ( vg_name)) == TRUE) {
       fprintf ( stderr, "%s -- volume group directory or file already exists\n"
                         "%s -- please choose a different name \n\n",
                         cmd, cmd);
       return LVM_EVGCREATE_VG_CHECK_DIR;
-   } 
+   }
 
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
@@ -270,7 +268,7 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- volume group \"%s\" already exists\n\n",
                         cmd, vg_name);
       return LVM_EVGCREATE_VG_CHECK_EXIST;
-   } 
+   }
 
    if ( opt_v > 0) printf ( "%s -- counting all existing volume groups\n", cmd);
    vg_name_ptr = lvm_tab_vg_check_exist_all_vg ();
@@ -281,15 +279,15 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- maximum volume group count of %d reached\n\n",
                         cmd, MAX_VG);
       return LVM_EVGCREATE_MAX_VG;
-   } 
+   }
 
    /* read all PVs */
    if ( opt_v > 0) printf ( "%s -- reading all physical volume data from "
                             "disks\n", cmd);
    if ( ( ret = pv_read_all_pv ( &pv, FALSE)) < 0) {
-	  fprintf ( stderr, "%s -- ERROR \"%s\" reading physical volumes\n\n",
+          fprintf ( stderr, "%s -- ERROR \"%s\" reading physical volumes\n\n",
                             cmd, lvm_error ( ret));
-	  return LVM_EVGCREATE_PV_READ_ALL_PV;
+          return LVM_EVGCREATE_PV_READ_ALL_PV;
    }
 
    /* check, if PVs are all defined and new */
@@ -326,7 +324,7 @@ int main ( int argc, char **argv) {
             if ( opt_v > 0) printf ( "%s -- checking for new physical "
                                      "volume \"%s\"\n", cmd, pv_name);
             if ( pv_check_new ( pv[p]) == FALSE) {
-               fprintf ( stderr, "%s -- \"%s\" is no new physical volume\n",
+               fprintf ( stderr, "%s -- \"%s\" is not a new physical volume\n",
 				 cmd, pv_name);
                if ( opt_v > 0) printf ( "%s -- checking volume group name "
                                         "of physical volume \"%s\"\n",
@@ -349,8 +347,8 @@ int main ( int argc, char **argv) {
                if ( strcmp ( pv_name, pvp[p1]->pv_name) == 0) {
                   fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" "
                                     "occurs multiple times\n\n",
-				    cmd, pv_name);
-	              return LVM_EVGCREATE_PV_MULTIPLE;
+                                   cmd, pv_name);
+                     return LVM_EVGCREATE_PV_MULTIPLE;
                }
             }
 
@@ -431,17 +429,27 @@ int main ( int argc, char **argv) {
                             cmd);
    if ( ( ret = vg_setup_for_create ( vg_name, &vg, pvp, pe_size,
                                       max_pv, max_lv)) < 0) {
-      if ( ret == -LVM_EVG_SETUP_FOR_CREATE_PV_SIZE) {
+      if ( ret == -LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MIN) {
          fprintf ( stderr, "%s -- ERROR: %d physical volume%s too small "
                            "for physical extent size of %s\n"
-                           "%s -- minimum physical volume size "
-                           "must be %s\n\n",
+                           "%s -- minimum physical volume "
+                           "at this physical extent size is %s\n\n",
                            cmd, count_sav, count_sav > 1 ? "s" : "",
                            vg_name, cmd,
-                           ( dummy = lvm_show_size ( vg.pe_size /
-                                                 2 * LVM_PE_SIZE_PV_SIZE_REL,
-                                                 SHORT)));
+                           ( dummy = lvm_show_size ( sectors_to_k(vg.pe_size) *
+			     LVM_PE_SIZE_PV_SIZE_REL,
+                             SHORT)));
                            free ( dummy); dummy = NULL;
+      } else if (ret == -LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MAX) {
+     	fprintf ( stderr, "%s -- ERROR: %d physical volume%s too large "
+			  "for physical extent size of %s\n"
+			  "%s -- maximum physical volume "
+			  "at this physical extent size is %s\n\n",
+			  cmd, count_sav, count_sav > 1 ? "s" : "",
+			  vg_name, cmd,
+			  ( dummy = lvm_show_size ( sectors_to_k(vg.pe_size) *
+			                            LVM_PE_T_MAX, SHORT)));
+			  free ( dummy); dummy = NULL;
       } else {
          fprintf ( stderr, "%s -- ERROR \"%s\" setting volume group \"%s\" up "
                            "for create\n\n",
@@ -507,7 +515,7 @@ int main ( int argc, char **argv) {
       }
    }
 
-   
+
    /* create VGDA in kernel */
    if ( opt_v > 0) printf ( "%s -- creating VGDA for volume group \"%s\" "
                             "in kernel\n",
