@@ -1,7 +1,7 @@
 /*
  * tools/vgremove.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * February, October 1997
  * May-June,September 1998
@@ -38,6 +38,7 @@
  *    25/09/1999 - added new exit codes
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
  *
  */
 
@@ -55,39 +56,26 @@ int main ( int argc, char **argv)
    int opt_v = 0;
    int p = 0;
    int ret = 0;
-#ifdef DEBUG
-   char *options = "dh?v";
-#else
-   char *options = "dh?v";
-#endif
+   char *options = "dh?v" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
-#ifdef DEBUG
-      { "debug",   no_argument, NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "help",    no_argument, NULL, 'h'},
       { "verbose", no_argument, NULL, 'v'},
       { NULL,      0,           NULL, 0},
    };
    char *vg_name = NULL;
    vg_t *vg = NULL;
+   pv_t **pv = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
                                long_options, NULL)) != EOF) {
       switch ( c) {
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
@@ -95,9 +83,7 @@ int main ( int argc, char **argv)
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
+                     DEBUG_HELP
                      "\t[-h/-?/--help]\n"
                      "\t[-v/--verbose]\n"
                      "\tVolumeGroupName [VolumeGroupName...]\n\n",
